@@ -2,19 +2,16 @@
 	Integrate.c
 		integrate over the unit hypercube
 		this file is part of Suave
-		last modified 2 Jan 08 th
+		last modified 16 Jun 10 th
 */
 
 
-static int Integrate(creal epsrel, creal epsabs,
-  cint flags, cnumber mineval, cnumber maxeval,
-  cnumber nnew, creal flatness,
-  real *integral, real *error, real *prob)
+static int Integrate(This *t, real *integral, real *error, real *prob)
 {
   TYPEDEFREGION;
 
   count dim, comp, df;
-  int fail = 1;
+  int fail = -99;
   Result totals[NCOMP];
   Region *anchor = NULL, *region = NULL;
 
@@ -23,26 +20,30 @@ static int Integrate(creal epsrel, creal epsabs,
     sprintf(s, "Suave input parameters:\n"
       "  ndim " COUNT "\n  ncomp " COUNT "\n"
       "  epsrel " REAL "\n  epsabs " REAL "\n"
-      "  flags %d\n  mineval " NUMBER "\n  maxeval " NUMBER "\n"
+      "  flags %d\n  seed %d\n"
+      "  mineval " NUMBER "\n  maxeval " NUMBER "\n"
       "  nnew " NUMBER "\n  flatness " REAL,
-      ndim_, ncomp_,
-      epsrel, epsabs,
-      flags, mineval, maxeval,
-      nnew, flatness);
+      t->ndim, t->ncomp,
+      t->epsrel, t->epsabs,
+      t->flags, t->seed,
+      t->mineval, t->maxeval,
+      t->nnew, t->flatness);
     Print(s);
   }
 
-#ifdef MLVERSION
-  if( setjmp(abort_) ) goto abort;
-#endif
+  if( BadComponent(t) ) return -2;
+  if( BadDimension(t) ) return -1;
 
-  IniRandom(2*maxeval, flags);
+  if( setjmp(t->abort) ) goto abort;
 
-  RegionAlloc(anchor, nnew, nnew);
+  t->epsabs = Max(t->epsabs, NOTZERO);
+  IniRandom(t);
+
+  RegionAlloc(anchor, t->nnew, t->nnew);
   anchor->next = NULL;
   anchor->div = 0;
 
-  for( dim = 0; dim < ndim_; ++dim ) {
+  for( dim = 0; dim < t->ndim; ++dim ) {
     Bounds *b = &anchor->bounds[dim];
     b->lower = 0;
     b->upper = 1;
@@ -57,12 +58,13 @@ static int Integrate(creal epsrel, creal epsabs,
     else Copy(b->grid, anchor->bounds[0].grid, NBINS);
   }
 
-  Sample(nnew, anchor, anchor->w,
-    anchor->w + nnew, anchor->w + (ndim_ + 1)*nnew, flags);
+  Sample(t, t->nnew, anchor, anchor->w,
+    anchor->w + t->nnew,
+    anchor->w + t->nnew + t->ndim*t->nnew);
   df = anchor->df;
   ResCopy(totals, anchor->result);
 
-  for( nregions_ = 1; ; ++nregions_ ) {
+  for( t->nregions = 1; ; ++t->nregions ) {
     Var var[NDIM][2], *vLR;
     real maxratio, maxerr, minfluct, bias, mid;
     Region *regionL, *regionR, *reg, **parent, **par;
@@ -76,9 +78,9 @@ static int Integrate(creal epsrel, creal epsabs,
 
       p += sprintf(p, "\n"
         "Iteration " COUNT ":  " NUMBER " integrand evaluations so far",
-        nregions_, neval_);
+        t->nregions, t->neval);
 
-      for( comp = 0; comp < ncomp_; ++comp ) {
+      for( comp = 0; comp < t->ncomp; ++comp ) {
         cResult *tot = &totals[comp];
         p += sprintf(p, "\n[" COUNT "] " 
           REAL " +- " REAL "  \tchisq " REAL " (" COUNT " df)",
@@ -90,7 +92,7 @@ static int Integrate(creal epsrel, creal epsabs,
 
     maxratio = -INFTY;
     maxcomp = 0;
-    for( comp = 0; comp < ncomp_; ++comp ) {
+    for( comp = 0; comp < t->ncomp; ++comp ) {
       creal ratio = totals[comp].err/MaxErr(totals[comp].avg);
       if( ratio > maxratio ) {
         maxratio = ratio;
@@ -98,12 +100,12 @@ static int Integrate(creal epsrel, creal epsabs,
       }
     }
 
-    if( maxratio <= 1 && neval_ >= mineval ) {
+    if( maxratio <= 1 && t->neval >= t->mineval ) {
       fail = 0;
       break;
     }
 
-    if( neval_ >= maxeval ) break;
+    if( t->neval >= t->maxeval ) break;
 
     maxerr = -INFTY;
     parent = &anchor;
@@ -117,15 +119,15 @@ static int Integrate(creal epsrel, creal epsabs,
       }
     }
 
-    Fluct(var[0], flatness,
+    Fluct(t, var[0],
       region->bounds, region->w, region->n, maxcomp,
-      region->result[maxcomp].avg, Max(maxerr, epsabs));
+      region->result[maxcomp].avg, Max(maxerr, t->epsabs));
 
-    bias = (epsrel < 1e-50) ? 2 :
-      Max(pow(2., -(real)region->div/ndim_)/epsrel, 2.);
+    bias = (t->epsrel < 1e-50) ? 2 :
+      Max(pow(2., -(real)region->div/t->ndim)/t->epsrel, 2.);
     minfluct = INFTY;
     bisectdim = 0;
-    for( dim = 0; dim < ndim_; ++dim ) {
+    for( dim = 0; dim < t->ndim; ++dim ) {
       cBounds *b = &region->bounds[dim];
       creal fluct = (var[dim][0].fluct + var[dim][1].fluct)*
         (bias - b->upper + b->lower);
@@ -138,10 +140,10 @@ static int Integrate(creal epsrel, creal epsabs,
     vLR = var[bisectdim];
     minfluct = vLR[0].fluct + vLR[1].fluct;
     nnewL = IMax(
-      (minfluct == 0) ? nnew/2 : (count)(vLR[0].fluct/minfluct*nnew),
+      (minfluct == 0) ? t->nnew/2 : (count)(vLR[0].fluct/minfluct*t->nnew),
       MINSAMPLES );
     nL = vLR[0].n + nnewL;
-    nnewR = IMax(nnew - nnewL, MINSAMPLES);
+    nnewR = IMax(t->nnew - nnewL, MINSAMPLES);
     nR = vLR[1].n + nnewR;
 
     RegionAlloc(regionL, nL, nnewL);
@@ -155,9 +157,9 @@ static int Integrate(creal epsrel, creal epsabs,
     bounds = &region->bounds[bisectdim];
     mid = bounds->mid;
     n = region->n;
-    w = wlast = region->w;  x = w + n;     f = flast = x + n*ndim_;
-    wL = regionL->w;        xL = wL + nL;  fL = xL + nL*ndim_;
-    wR = regionR->w;        xR = wR + nR;  fR = xR + nR*ndim_;
+    w = wlast = region->w;  x = w + n;     f = flast = x + n*t->ndim;
+    wL = regionL->w;        xL = wL + nL;  fL = xL + nL*t->ndim;
+    wR = regionR->w;        xR = wR + nR;  fR = xR + nR*t->ndim;
 
     while( n-- ) {
       cbool final = (*w < 0);
@@ -165,24 +167,24 @@ static int Integrate(creal epsrel, creal epsabs,
         if( final && wR > regionR->w ) *(wR - 1) = -fabs(*(wR - 1));
         *wL++ = *w++;
         VecCopy(xL, x);
-        xL += ndim_;
+        xL += t->ndim;
         ResCopy(fL, f);
-        fL += ncomp_;
+        fL += t->ncomp;
       }
       else {
         if( final && wL > regionL->w ) *(wL - 1) = -fabs(*(wL - 1));
         *wR++ = *w++;
         VecCopy(xR, x);
-        xR += ndim_;
+        xR += t->ndim;
         ResCopy(fR, f);
-        fR += ncomp_;
+        fR += t->ncomp;
       }
-      x += ndim_;
-      f += ncomp_;
+      x += t->ndim;
+      f += t->ncomp;
       if( n && final ) wlast = w, flast = f;
     }
 
-    Reweight(region->bounds, wlast, flast, f, totals, flags);
+    Reweight(t, region->bounds, wlast, flast, f, totals);
     VecCopy(regionL->bounds, region->bounds);
     VecCopy(regionR->bounds, region->bounds);
 
@@ -193,12 +195,12 @@ static int Integrate(creal epsrel, creal epsabs,
 
     StretchGrid(bounds->grid, boundsL->grid, boundsR->grid);
 
-    Sample(nnewL, regionL, wL, xL, fL, flags);
-    Sample(nnewR, regionR, wR, xR, fR, flags);
+    Sample(t, nnewL, regionL, wL, xL, fL);
+    Sample(t, nnewR, regionR, wR, xR, fR);
 
     df += regionL->df + regionR->df - region->df;
 
-    for( comp = 0; comp < ncomp_; ++comp ) {
+    for( comp = 0; comp < t->ncomp; ++comp ) {
       cResult *r = &region->result[comp];
       Result *rL = &regionL->result[comp];
       Result *rR = &regionR->result[comp];
@@ -227,7 +229,7 @@ static int Integrate(creal epsrel, creal epsabs,
     region = NULL;
   }
 
-  for( comp = 0; comp < ncomp_; ++comp ) {
+  for( comp = 0; comp < t->ncomp; ++comp ) {
     cResult *tot = &totals[comp];
     integral[comp] = tot->avg;
     error[comp] = tot->err;
@@ -237,22 +239,22 @@ static int Integrate(creal epsrel, creal epsabs,
 #ifdef MLVERSION
   if( REGIONS ) {
     MLPutFunction(stdlink, "List", 2);
-    MLPutFunction(stdlink, "List", nregions_);
+    MLPutFunction(stdlink, "List", t->nregions);
     for( region = anchor; region; region = region->next ) {
       real lower[NDIM], upper[NDIM];
 
-      for( dim = 0; dim < ndim_; ++dim ) {
+      for( dim = 0; dim < t->ndim; ++dim ) {
         cBounds *b = &region->bounds[dim];
         lower[dim] = b->lower;
         upper[dim] = b->upper;
       }
 
       MLPutFunction(stdlink, "Cuba`Suave`region", 4);
-      MLPutRealList(stdlink, lower, ndim_);
-      MLPutRealList(stdlink, upper, ndim_);
+      MLPutRealList(stdlink, lower, t->ndim);
+      MLPutRealList(stdlink, upper, t->ndim);
 
-      MLPutFunction(stdlink, "List", ncomp_);
-      for( comp = 0; comp < ncomp_; ++comp ) {
+      MLPutFunction(stdlink, "List", t->ncomp);
+      for( comp = 0; comp < t->ncomp; ++comp ) {
         cResult *r = &region->result[comp];
         real res[] = {r->avg, r->err, r->chisq};
         MLPutRealList(stdlink, res, Elements(res));
@@ -263,11 +265,8 @@ static int Integrate(creal epsrel, creal epsabs,
   }
 #endif
 
-#ifdef MLVERSION
 abort:
-#endif
-
-  if( region ) free(region);
+  free(region);
 
   while( (region = anchor) ) {
     anchor = anchor->next;
